package com.ruoyi.system.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.ruoyi.common.constant.IotConstants;
import com.ruoyi.common.constant.ServiceConstants;
import com.ruoyi.common.core.domain.entity.RlUser;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.system.domain.deliveryOrder.RlDeliveryOrder;
import com.ruoyi.system.domain.deliveryOrder.RlDeliveryOrderVO;
import com.ruoyi.system.domain.deliveryOrderOper.RlDeliveryOrderOper;
import com.ruoyi.system.domain.order.RlOrder;
import com.ruoyi.system.domain.order.RlOrderVO;
import com.ruoyi.system.mapper.RlDeliveryOrderMapper;
import com.ruoyi.system.service.IRlDeliveryOrderOperService;
import com.ruoyi.system.service.IRlDeliveryOrderService;
import com.ruoyi.system.service.IRlOrderService;
import com.ruoyi.system.service.IRlUserService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.support.TransactionTemplate;

import javax.annotation.Resource;
import java.util.List;

/**
 * 配送工单Service业务层处理
 *
 * @author qzz
 * @date 2024-10-10
 */
@Slf4j
@Service
public class RlDeliveryOrderServiceImpl implements IRlDeliveryOrderService
{
    @Resource
    private RlDeliveryOrderMapper rlDeliveryOrderMapper;

    @Autowired
    private IRlOrderService orderService;

    @Autowired
    private IRlUserService userService;

    @Autowired
    private IRlDeliveryOrderOperService deliveryOrderOperService;

    @Autowired
    private TransactionTemplate transactionTemplate;


    /**
     * 查询配送工单
     *
     * @param deliveryId 配送工单主键
     * @return 配送工单
     */
    @Override
    public RlDeliveryOrder selectRlDeliveryOrderByDeliveryId(Long deliveryId)
    {
        return rlDeliveryOrderMapper.selectRlDeliveryOrderByDeliveryId(deliveryId);
    }

    /**
     * 根据订单号查询配送工单
     *
     * @param orderNo 订单号
     * @return 配送工单
     */
    @Override
    public RlDeliveryOrder selectRlDeliveryOrderByOrderNo(String orderNo) {
        return rlDeliveryOrderMapper.selectRlDeliveryOrderByOrderNo(orderNo);
    }

    /**
     * 查询配送工单列表
     *
     * @param rlDeliveryOrder 配送工单
     * @return 配送工单
     */
    @Override
    public List<RlDeliveryOrderVO> selectRlDeliveryOrderList(RlDeliveryOrder rlDeliveryOrder)
    {
        return rlDeliveryOrderMapper.selectRlDeliveryOrderList(rlDeliveryOrder);
    }

    /**
     * 新增配送工单
     *
     * @param rlDeliveryOrder 配送工单
     * @return 结果
     */
    @Override
    public int insertRlDeliveryOrder(RlDeliveryOrder rlDeliveryOrder)
    {
        rlDeliveryOrder.setCreateTime(DateUtils.getNowDate());
        return rlDeliveryOrderMapper.insertRlDeliveryOrder(rlDeliveryOrder);
    }

    /**
     * 修改配送工单
     *
     * @param rlDeliveryOrder 配送工单
     * @return 结果
     */
    @Override
    public int updateRlDeliveryOrder(RlDeliveryOrder rlDeliveryOrder)
    {
        return rlDeliveryOrderMapper.updateRlDeliveryOrder(rlDeliveryOrder);
    }

    /**
     * 批量删除配送工单
     *
     * @param deliveryIds 需要删除的配送工单主键
     * @return 结果
     */
    @Override
    public int deleteRlDeliveryOrderByDeliveryIds(Long[] deliveryIds)
    {
        return rlDeliveryOrderMapper.deleteRlDeliveryOrderByDeliveryIds(deliveryIds);
    }

    /**
     * 删除配送工单信息
     *
     * @param deliveryId 配送工单主键
     * @return 结果
     */
    @Override
    public int deleteRlDeliveryOrderByDeliveryId(Long deliveryId)
    {
        return rlDeliveryOrderMapper.deleteRlDeliveryOrderByDeliveryId(deliveryId);
    }

    /**
     * 立即接单
     *    1.状态变成待配送
     *    2.绑定配送员
     *    3.添加工单履历
     */
    @Override
    public int orderTaking(String orderNo, Long userId) {
        /** 1.状态变成待配送 */
        RlDeliveryOrder deliveryOrder = rlDeliveryOrderMapper.selectRlDeliveryOrderByOrderNo(orderNo);
        if(ObjectUtil.isNull(deliveryOrder)){
            throw new ServiceException("配送单不存在");
        }
        if(!deliveryOrder.getStatus().equals(ServiceConstants.DELIVERY_ORDER_STATUS_PENDING)){
            throw new ServiceException("配送状态不是待接单");
        }
        RlUser rlUser = userService.selectUserById(userId);
        RlDeliveryOrder updateDeliveryOrder = new RlDeliveryOrder();
        updateDeliveryOrder.setDeliveryId(deliveryOrder.getDeliveryId());
        updateDeliveryOrder.setStatus(ServiceConstants.DELIVERY_ORDER_STATUS_WAITING_DELIVERY);
        updateDeliveryOrder.setDeliverymanId(userId);
        updateDeliveryOrder.setDeliveryman(rlUser.getUserName());
        updateDeliveryOrder.setUpdateTime(DateUtils.getNowDate());
        updateDeliveryOrder.setDeliverymanPhone(rlUser.getPhonenumber());
        String deliverymanType = null;
        if (rlUser.getUserType().equals(ServiceConstants.USER_TYPE_MERCHANT)) {
            deliverymanType = ServiceConstants.DELIVERY_PERSON_TYPE_MERCHANT;
        } else if (rlUser.getUserType().equals(ServiceConstants.USER_TYPE_DISPATCHER)) {
            deliverymanType = ServiceConstants.DELIVERY_PERSON_TYPE_COURIER;
        }
        updateDeliveryOrder.setDeliverymanType(deliverymanType);

        Boolean execute = transactionTemplate.execute(e -> {
            int i = rlDeliveryOrderMapper.updateRlDeliveryOrder(updateDeliveryOrder);
            if(i==0){
                log.error("【立即接单】更新配送单失败");
                throw new ServiceException("【立即接单】更新配送单失败");
            }
            /** 3.添加工单履历 */
            int i2 =  deliveryOrderOperService.deliveryOrderOperLog(userId, rlUser.getUserName(),rlUser.getPhonenumber(),updateDeliveryOrder.getDeliveryId(),ServiceConstants.DELIVERY_TYPE_ACCEPT_ORDER);
            if(i2==0){
                log.error("【立即接单】添加工单履历失败");
                throw new ServiceException("【立即接单】添加工单履历失败");
            }
            return Boolean.TRUE;
        });
        if(!execute)throw new ServiceException("接单失败");
        return 1;
    }

    /**
     * 立即配送
     *    1.状态变成配送中
     *    3.添加工单履历
     */
    @Override
    public int inDelivery(String orderNo, Long userId) {
        /** 1.状态变成配送中 */
        RlDeliveryOrder deliveryOrder = rlDeliveryOrderMapper.selectRlDeliveryOrderByOrderNo(orderNo);
        if(ObjectUtil.isNull(deliveryOrder)){
            throw new ServiceException("配送单不存在");
        }
        if(!deliveryOrder.getStatus().equals(ServiceConstants.DELIVERY_ORDER_STATUS_WAITING_DELIVERY)){
            throw new ServiceException("配送状态不是待配送");
        }
        RlUser rlUser = userService.selectUserById(userId);
        RlDeliveryOrder updateDeliveryOrder = new RlDeliveryOrder();
        updateDeliveryOrder.setDeliveryId(deliveryOrder.getDeliveryId());
        updateDeliveryOrder.setStatus(ServiceConstants.DELIVERY_ORDER_STATUS_IN_DELIVERY);
        updateDeliveryOrder.setUpdateTime(DateUtils.getNowDate());

        Boolean execute = transactionTemplate.execute(e -> {
            int i = rlDeliveryOrderMapper.updateRlDeliveryOrder(updateDeliveryOrder);
            if(i==0){
                log.error("【立即配送】更新配送单失败");
                throw new ServiceException("【立即配送】更新配送单失败");
            }
            /** 3.添加工单履历 */
            int i2 =  deliveryOrderOperService.deliveryOrderOperLog(userId, rlUser.getUserName(),rlUser.getPhonenumber(),updateDeliveryOrder.getDeliveryId(),ServiceConstants.DELIVERY_TYPE_DELIVERING);
            if(i2==0){
                log.error("【立即配送】添加工单履历失败");
                throw new ServiceException("【立即配送】添加工单履历失败");
            }
            return Boolean.TRUE;
        });
        if(!execute)throw new ServiceException("配送失败");
        return 1;
    }

    /**
     * 结束配送
     *    1.状态变成已完成
     *    3.添加工单履历
     */
    @Override
    public int endDelivery(String orderNo, Long userId) {
        /** 1.状态变成已完成 */
        RlDeliveryOrder deliveryOrder = rlDeliveryOrderMapper.selectRlDeliveryOrderByOrderNo(orderNo);
        if(ObjectUtil.isNull(deliveryOrder)){
            throw new ServiceException("配送单不存在");
        }
        if(!deliveryOrder.getStatus().equals(ServiceConstants.DELIVERY_ORDER_STATUS_IN_DELIVERY)){
            throw new ServiceException("配送状态不是配送中");
        }
        RlUser rlUser = userService.selectUserById(userId);
        RlDeliveryOrder updateDeliveryOrder = new RlDeliveryOrder();
        updateDeliveryOrder.setDeliveryId(deliveryOrder.getDeliveryId());
        updateDeliveryOrder.setStatus(ServiceConstants.DELIVERY_ORDER_STATUS_COMPLETED);
        updateDeliveryOrder.setUpdateTime(DateUtils.getNowDate());

        Boolean execute = transactionTemplate.execute(e -> {
            int i = rlDeliveryOrderMapper.updateRlDeliveryOrder(updateDeliveryOrder);
            if(i==0){
                log.error("【结束配送】更新配送单失败");
                throw new ServiceException("【结束配送】更新配送单失败");
            }
            /** 3.添加工单履历 */
            int i2 =  deliveryOrderOperService.deliveryOrderOperLog(userId, rlUser.getUserName(),rlUser.getPhonenumber(),updateDeliveryOrder.getDeliveryId(),ServiceConstants.DELIVERY_TYPE_FINISHED);
            if(i2==0){
                log.error("【结束配送】添加工单履历失败");
                throw new ServiceException("【结束配送】添加工单履历失败");
            }
            return Boolean.TRUE;
        });
        if(!execute)throw new ServiceException("结束配送失败");
        return 1;
    }

    /**
     * 取消配送
     *    1.状态变成已取消
     *    3.添加工单履历
     */
    @Override
    public int cancelDelivery(String orderNo, Long userId) {
        /** 1.状态变成已完成 */
        RlDeliveryOrder deliveryOrder = rlDeliveryOrderMapper.selectRlDeliveryOrderByOrderNo(orderNo);
        if(ObjectUtil.isNull(deliveryOrder)){
            throw new ServiceException("配送单不存在");
        }
        RlUser rlUser = userService.selectUserById(userId);
        RlDeliveryOrder updateDeliveryOrder = new RlDeliveryOrder();
        updateDeliveryOrder.setDeliveryId(deliveryOrder.getDeliveryId());
        updateDeliveryOrder.setStatus(ServiceConstants.DELIVERY_ORDER_STATUS_CANCELLED);
        updateDeliveryOrder.setUpdateTime(DateUtils.getNowDate());

        Boolean execute = transactionTemplate.execute(e -> {
            int i = rlDeliveryOrderMapper.updateRlDeliveryOrder(updateDeliveryOrder);
            if(i==0){
                log.error("【取消配送】更新配送单失败");
                throw new ServiceException("【取消配送】更新配送单失败");
            }
            /** 3.添加工单履历 */
            int i2 =  deliveryOrderOperService.deliveryOrderOperLog(userId, rlUser.getUserName(),rlUser.getPhonenumber(),updateDeliveryOrder.getDeliveryId(),ServiceConstants.DELIVERY_TYPE_CANCEL);
            if(i2==0){
                log.error("【取消配送】添加工单履历失败");
                throw new ServiceException("【取消配送】添加工单履历失败");
            }
            return Boolean.TRUE;
        });
        if(!execute)throw new ServiceException("取消配送失败");
        return 1;
    }



}
